# -*- coding: iso-8859-1 -*-
#
# $Id: MultipleChoiceTool.py,v 1.4 2005/05/20 10:11:15 mxp Exp $
#
# Copyright  2004 Otto-von-Guericke-Universitt Magdeburg
#
# This file is part of LlsMultipleChoice.
#
# LlsMultipleChoice is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LlsMultipleChoice is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LlsMultipleChoice; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""MultipleChoiceTool main class"""

# Zope stuff
from Globals import InitializeClass 
from AccessControl import ClassSecurityInfo
from OFS.Folder import Folder

# CMF stuff
from Products.CMFCore.Expression import Expression
from Products.CMFCore.ActionInformation import ActionInformation
from Products.CMFCore.ActionProviderBase import ActionProviderBase
from Products.CMFCore.CMFCorePermissions import View, ManagePortal
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.utils import UniqueObject

# My stuff ;-)
#~ from MultipleChoiceParser import ParserFolder
#~ from MultipleChoiceRenderer import RendererFolder

from Products.LlsMultipleChoice.tools import log


class MultipleChoiceTool(UniqueObject, Folder, ActionProviderBase):
    """
        Tool for managing import and export functionality
        as well as some resources of the BibliographyFolders
        and -Entries.
    """

    id = 'portal_multiplechoice'
    meta_type = 'Multiple Choice Tool'
    _actions = (ActionInformation(
            id='multiplechoice_view', 
            title='Multiple Choice',
            description='Site wide; sorted by year and authors',
            action=Expression(
            text='string: ${portal_url}/multiplechoicetool_view'),
            permissions=(View,),
            category='portal_tabs',
            condition=None,
            visible=1
            ),
        )

    security = ClassSecurityInfo()
    security.declareObjectProtected(View)
    
    manage_options = (
            (Folder.manage_options[0],)
            + ActionProviderBase.manage_options
            + Folder.manage_options[2:]
        )

    _properties = Folder._properties + (
##         {'id':'reference_types',
##          'type':'lines',
##          'mode':'w',
##          },
        {'id':'preprint_servers',
         'type':'lines',
         'mode':'w',
         },
        )

    def __init__(self):
        #~ self._setObject('Parsers', ParserFolder('Parsers', ''))
        #~ self._setObject('Renderers', RendererFolder('Renderers', ''))
        return


    #~ security.declarePublic('getReferenceTypes')
    #~ def getReferenceTypes(self):
        #~ """
        #~ returns a list with the names (meta types) of the
        #~ currently allowed reference types of a BibliographyFolder
        #~ """
        #~ types_tool = getToolByName(self, 'portal_types')
        #~ return types_tool.BibliographyFolder.allowed_content_types
    
    security.declarePublic('getUploadFormatNames')
    def getUploadFormatNames(self):
        """
        returns a list with the names of the supported upload formats
        """
        return [parser.getFormatName() \
                for parser in self.Parsers.objectValues()]

    security.declarePublic('getUploadFormatExtensions')
    def getUploadFormatExtensions(self):
        """
        returns a list with the file name extensions
        of the supported upload formats
        """
        return [parser.getFormatExtension() \
                for parser in self.Parsers.objectValues()]
    

    security.declarePublic('getDownloadFormatNames')
    def getDownloadFormatNames(self):
        """
        returns a list with the names of the supported download formats
        """
        return [renderer.getFormatName() \
                for renderer in self.Renderers.objectValues()]

    security.declarePublic('getDownloadFormatExtensions')
    def getDownloadFormatExtensions(self):
        """
        returns a list with the file name extensions
        of the supported download formats
        """
        return [renderer.getFormatExtension() \
                for renderer in self.Renderers.objectValues()]

    security.declarePublic('getDownloadFormats')
    def getDownloadFormats(self):
        """
        returns a list of (name, extension) tuples
        of the supported download formats
        """
        return zip(self.getDownloadFormatNames(),
                   self.getDownloadFormatExtensions())

    def render(self, entry, format):
        """
        renders a BibliographyEntry object in the specified format
        """
        renderer = self.getRenderer(format)

        if renderer:
            return renderer.render(entry)
        else:
            return None

    def getRenderer(self, format):
        """
        returns the renderer for the specified format
        first looks for a renderer with the 'format' name
        next looks for a renderer with the 'format' extension
        """
        for renderer in self.Renderers.objectValues():
            if format.lower() == renderer.getFormatName().lower():
                return renderer
            if format.lower() == renderer.getFormatExtension().lower():
                return renderer
        return None
    
    #~ def getEntries(self, source, format, file_name=None):
        #~ """
        #~ main routine to be called from BibliographyFolders
        #~ returns a list with the parsed entries
        #~ """
        #~ format = self.checkFormat(source, format, file_name)
        
        #~ parser = self.getParser(format)

        #~ if parser:
            #~ return parser.getEntries(source)
        #~ else:
            #~ return "No parser for '%s' available." % format

    def getParser(self, format):
        """
        returns the parser for the specified format
        first looks for a parser with the 'format' name
        next looks for a parser with the 'format' extension
        """
        for parser in self.Parsers.objectValues():
            if format.lower() == parser.getFormatName().lower():
                return parser
            elif format.lower() == parser.getFormatExtension().lower():
                return parser
        return None ## rr: we probabliy should raise an error here 

    def checkFormat(self, source, format, file_name):
        """
        plausibility test whether 'source' has the 'format' specified
        if not it tries to infer the format from the 'file_name'
        raises an error if both fail
        """
        ok = self.getParser(format).checkFormat(source)
        if not ok and file_name:
            format = self.guessFormat(file_name)
        if format:
            return format
        else:
            raise  ## rr: the type of error still needs to be defined

    def guessFormat(self, file_name):
        """
        Checks whether the file_name extension is
        among the supported ones.

        returns the respective format name if found 
        returns None otherwise
        """
        extension = file_name.split('.')[-1].lower()
        if extension in [ext.lower()
                         for ext in self.getUploadFormatExtensions()]:
            for parser in self.Parsers.objectValues():
                if extension == parser.getFormatExtension().lower():
                    return parser.getFormatName()
        return None

try:
    InitializeClass(MultipleChoiceTool)
    log('Worked: "InitializeClass(MultipleChoiceTool)"\n')
except Exception, e:
    log('Failed: "InitializeClass(MultipleChoiceTool)": ' + unicode(e) + '\n')
