# -*- coding: iso-8859-1 -*-
#
# $Id: QuestionGroup.py,v 1.13 2005/10/21 13:46:13 mxp Exp $
#
# Copyright  2004 Otto-von-Guericke-Universitt Magdeburg
#
# This file is part of LlsMultipleChoice.
#
# LlsMultipleChoice is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LlsMultipleChoice is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LlsMultipleChoice; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import random

from AccessControl import ClassSecurityInfo

from Products.Archetypes.public import Schema, BooleanField, StringField, StringWidget, TextField, TextAreaWidget, registerType

from Products.LlsMultipleChoice.config import PROJECTNAME, SKINS_DIR, \
    QUESTION_DIR, QUESTION_TYPES, ANSWER_DIR, \
    ANSWER_TYPES, GLOBALS, PERMISSION_INTERROGATOR, \
    PERMISSION_STUDENT, CUSTOM_EVALUATION_FUNCTION_NAME, I18N_DOMAIN
from Products.LlsMultipleChoice.tools import getParent, log, isNumeric
from Products.LlsMultipleChoice.AbstractQuestionContainer import AbstractQuestionContainer

class QuestionGroup(AbstractQuestionContainer):
    """
    """
    
    # See comments in 'AbstractQuestionContainer' for an explanation
    # of the function of a schema, therein defined properties (fields) and 
    # internationalization of the widgets
    schema = AbstractQuestionContainer.schema + Schema((
            StringField('title',
                required=False,
                searchable=True,
                default='',
                widget=StringWidget(
                    label_msgid='label_title',
                    description_msgid='title',
                    i18n_domain='plone'),
            ),
        ),)

    meta_type = portal_type = archetype_name = 'MC Question Group'
    content_icon = 'QuestionGroup.png'
    
    security = ClassSecurityInfo()
    
    # The inherited method 'contentValues()' may not be called by just anoybody, directly or indirectly.
    security.declareProtected(PERMISSION_INTERROGATOR, 'contentValues')    
    
    # This function has to be implemented by classes derived from AbstractQuestionContainer
    def getResults(self):
        """ See 'MultipleChoiceTest' """
        return getParent(self).getResults()
        
    # This function has to be implemented by classes derived from AbstractQuestionContainer
    def setResults(self, results = {}):
        """ See 'MultipleChoiceTest' """
        return getParent(self).setResults(results)
    
    # This function has to be implemented by classes derived from AbstractQuestionContainer
    def isPublic(self):
        """ See 'MultipleChoiceTest' """
        return getParent(self).isPublic()
        
    # This function has to be implemented by classes derived from AbstractQuestionContainer
    def getEvaluationScripts(self):
        """ See 'MultipleChoiceTest' """
        return getParent(self).getEvaluationScripts()
        
    # This function has to be implemented by classes derived from AbstractQuestionContainer
    #~ def getAllQuestions(self):
        #~ """ Return all the question objects contaoned in this group. """
        #~ return self.listFolderContents()
        

    def getCandidatePoints(self, candidateId):
        """ Return how many points the candidate got for the questions in this container.
              If a custom evaluation script has been uploaded it will be
            invoked. Otherwise a default method will be used. 
            
            @param candidateId The user ID of the candidate whose points you want to know. """
        customScript = self.getEvaluationScript(self.archetype_name)
        questions = self.getQuestions(candidateId)
        if(not customScript): # default
            points = 0
            for question in questions:
                if(not ('getCandidatePoints' in dir(question))):
                    return None
                questionCandidatePoints = question.getCandidatePoints(candidateId)
                if(not isNumeric(questionCandidatePoints)):
                    return None
                points = points + questionCandidatePoints
            return points
        else: # custom
            return evalFunString(customScript, CUSTOM_EVALUATION_FUNCTION_NAME, \
                [self, candidateId, questions])
        
        
# Register this type in Zope
try:
    registerType(QuestionGroup)
    log('Worked: registerType(QuestionGroup)\n')
except Exception, e:
    log('Failed: registerType(QuestionGroup): ' + unicode(e) + '\n')
    raise e
